.TH RECFUNK_ASCII V1.0 10/03/2001
.SH NAME
\fIRECFUNK_ASCII\fR - a FORTRAN code that will transform a collection of 3 component records 
(vertical, radial, transverse) into ASCII tables of 2 component RFs (radial and transverse), binned with backazimuth or epicentral distance.

.SH SYNOPSIS
Run \fIrecfunk_ascii\fR from the command line. The code prompts user for input values,
and dumps intermediate output to the screen.
Alternatively, \fIrecfunk_ascii < runfile > logfile\fR, where \fIrunfile\fR contains
required run parameters (max frequency, file with input data parameters, range limits for stacking, bin widths etc.).


.SH DESCRIPTION

The code reads a file of data filenames and time intervals (see below).
The data must be in SAC format, all sampled at the same rate, with horizontals rotated to radial and transverse.
The code writes the BAZ- and EPICEN-dependent RFs to ASCII tables with fields: time, bin (epicentral or backazimuthal),  value. Output RFs for individual bins are separated by ">" lines, in anticipation of using GMT to plot them.
Output filenames are out[rt]_baz.grid out[rt]_epi.grid. These files are overwritten everytime you run the program, so rename them to save them.
Definition of ranges for RF stacking is up to the user.
Step of backazimuthal binning is hard-wired to 10 degrees. Step of the epicentral distance binning defaults to 10 degrees, but may be changed through
during the run. Bins are always overlapped by 50%.

.SH USAGE

In the course of the run, the user has to provide, by typing or via a "runfile",
the following parameters:
.sp
maximum frequency - computations will be performed up to this frequency.
A cosine-squared taper [ (cos(pi*f/(2*f_max))**2 ] is used to smooth the spectra, so effective spectral content will have frequencies up to f_max/2.
.sp
name of the ASCII file with data filenames and processing parameters. See below for details.
.sp
epicentral range half-width in degrees. 
EXAMPLE:  20 --> stack in each back-azimuth bin will be limited
to epicentral distances +-20 degrees from the epicentral distance
with most data traces.
 Giving values of 0 or 180 results in stacking 
of all events within the backazimuthal bin.
.sp
back-azimuth limits for an epicentral-distance binned RFs. 
EXAMPLE:  30 120 --> make epicentral sweep of RFs restricted to
30< BAZ <120
The whole 360 deg range will be stacked if the same value is given for both limits, and a 360-wraparound is performed if limit1 < limit2.
.sp
epicentral bin control flag:
EXAMPLE:
1  --> user-defined epicentral binning
10 40 3  --> epicentral bins of halfwidth 3 degrees centered at
epicentral distances 10, 13, 16, ... 40 degrees (right?)

.SH INPUT DATA REQUIREMENTS
SAC formatted records of 3 components of motion (Z,R,T) are needed for
the computation of RFs. Headers of SAC-formatted files need to contain
receiver and source positions, and fields "gcarc" and "baz" need to be
filled in. The time window for analysis needs to have equal amounts of
pre-event noise and signal, e.g., 100 sec of each. Analysis windows are
defined explicitly for each triplet of records in the input file
"in_recfunk".
Naming convention for data files is as follows:
.br
3 files are expected, with vertical, radial and transverse components of motion.
File names have to end with either r,t or z. 

.SH INPUT FILE FORMAT

The input file name (mandatory!) is "in_recfunk" and has lines of the form: 
 
 1997.070.19.33.bh? <-- code replaces ? with r,t,z 
.br
 57 52 <-- start time of analysis window, window duration (sec) 
.br
 1997.076.08.15.bh?
.br 
 62 62 
.br
 ... 
.br
 stop <-- tells code that data is finished, 599 events max 

.SH OUTPUT FILES
The code creates four output files, creatively named out[rt]_baz.grid out[rt]_epi.grid. The format is ASCII, 3-column tables,
with values:
.br
time_step	bin_center (degrees)	RF_value
.br
These files will be written over every time.

.SH SEE ALSO
\fIrecfunk\fR - prototype code written by J. Park,
at www.geology.yale.edu/~park/freecodes.html.
.sp
General ideology of the computations is outlined in 

Park, J., and V. Levin, Receiver functions from
 multiple-taper spectral correlation estimates,
 BSSA, v90, pp1507-1520, 2000

.sp
a website www.ldeo.columbia.edu/~vadim/RF/RF-manual.html provides a practical guide for this and related software.
