.TH RFSYN V1.0 10/03/2001
.SH NAME
\fIrfsyn\fR - FORTRAN code computing a receiver-function response
of a
stack of anisotropic layers to a plane wave incident from below
via a
reflectivity algorithm.
.SH SYNOPSIS
Run \fIrfsyn\fR from the command line. The code prompts user for
input values,
and dumps intermediate output to the screen.
Alternatively, \fIrfsyn < runfile > logfile\fR, where
\fIrunfile\fR contains
required run parameters (max frequency, backazimuth, model name,
phase velocity).


.SH DESCRIPTION
The code reads a 1D model of velocity, density and anisotropy
layers with depth, computes 3 components of motion for the
response of this model to an incident
compressional pulse.
On output, the code computes receiver functions from the
division of smoothed spectra of respective components.
Adjusts for the phase velocity and BAZ of the incident
compressional phase.
Computes waveforms up to a user-specified frequency.

.SH USAGE
Input parameters required:
.sp
maximum frequency - computations will be performed up to this
frequency.
A cosine-squared taper [ (cos(pi*f/(2*f_max))**2 ] is used to
smooth the spectra, so effective spectral content will have
frequencies up to f_max/2.

.sp
backazimuth - direction from which the "parent" P wave comes.
This parameter
matters if the model is anisotropic.
.sp
name of the file with model description - see below for
model format
.sp
phase velocity - a proxy for the incidence angle of the incoming
P wave.
Phase velocities in IASPEI91 model for a source at depth of 33
km:
      30 deg - 12.6 km/s
      60 deg - 16.3 km/s
      90 deg - 23.88 km/s
      120 deg - 58.56 km/s (PKP)
.sp
Output goes to files \fBRRF.table\fR and \fBTRF.table\fR (the
names are hardwired), in the form of ASCII tables with 3 columns
(time,
BAZ, RF_value).
.SH MODEL FILE FORMAT

AAK model <--- any string
.br
5 <--- number of layers not counting last
.br
0 0 <--- tilt from vertical and azimuth from north for the
anisotropic symmetry axis
.br
700 3500 0.00 0.00 2000 0.00 2000 <--- bottom of the layer,
Vp(m/s), B, C, Vs(m/s), E, rho (density, kg/m^3)
.br
50 200
.br
2000 4500 -0.10 0.00 2600 -0.10 2100
.br
0 0
.br
9000 5800 0.00 0.00 3350 0.00 2100
.br
70 10
.br
23000 6500 0.00 0.00 3400 0.00 2600
.br
0 0
.br
55000 6900 0.00 0.00 3900 0.00 2800
.br
0 0
.br
90000 8100 0.00 0.00 4500 0.00 3300 <---- LAST "LAYER" simulates
the halfspace, HAS TO BE ISOTROPIC!!!!

.SH SEE ALSO
\fIanirec\fR - prototype code written by J. Park,
at www.geology.yale.edu/~park/freecodes.html.
.sp
General ideology of the computations is outlined in Levin & Park,
GJI v131, pp 253-266, 1997.


